@@ -298,13 +298,13 @@ def one_of(args):
     pass  # pragma: no cover
 
 
-@overload
+@overload  # noqa: F811
 def one_of(*args):
     # type: (SearchStrategy[Any]) -> SearchStrategy[Any]
     pass  # pragma: no cover
 
 
-def one_of(*args):
+def one_of(*args):  # noqa: F811
     # Mypy workaround alert:  Any is too loose above; the return parameter
     # should be the union of the input parameters.  Unfortunately, Mypy <=0.600
     # raises errors due to incompatible inputs instead.  See #1270 for links.
@@ -669,14 +669,14 @@ def sampled_from(elements):
     pass  # pragma: no cover
 
 
-@overload
+@overload  # noqa: F811
 def sampled_from(elements):
     # type: (Type[enum.Enum]) -> SearchStrategy[Any]
     # `SearchStrategy[Enum]` is unreliable due to metaclass issues.
     pass  # pragma: no cover
 
 
-@defines_strategy
+@defines_strategy  # noqa: F811
 def sampled_from(elements):
     """Returns a strategy which generates any value present in ``elements``.
 
@@ -1017,7 +1017,7 @@ def characters(
             "Nothing is excluded by other arguments, so passing only "
             "whitelist_characters=%(chars)r would have no effect.  Also pass "
             "whitelist_categories=(), or use sampled_from(%(chars)r) instead."
-            % dict(chars=whitelist_characters)
+            % {"chars": whitelist_characters}
         )
     blacklist_characters = blacklist_characters or ""
     whitelist_characters = whitelist_characters or ""
@@ -2017,7 +2017,7 @@ def complex_numbers(
             "Cannot have allow_nan=%r, min_magnitude=%r max_magnitude=%r"
             % (allow_nan, min_magnitude, max_magnitude)
         )
-    allow_kw = dict(allow_nan=allow_nan, allow_infinity=allow_infinity)
+    allow_kw = {"allow_nan": allow_nan, "allow_infinity": allow_infinity}
 
     if min_magnitude is None and max_magnitude is None:
         # In this simple but common case, there are no constraints on the
